'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Parallel2 = require('./Parallel');

var _Parallel3 = _interopRequireDefault(_Parallel2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Limiter
 *
 * Parallel with limit for max running tasks
 *
 * @param {number} limit max running tasks
 * @param {Function} complete function to be called on complete
 * @param {Function} recover function to be called on recover
 * @param {boolean} completeOnFirst completes on last or on first
 */
var Limiter = function (_Parallel) {
  _inherits(Limiter, _Parallel);

  /**
   * Limiter constructor
   *
   * @param {number} limit max running tasks
   * @param {Function} complete function to be called on complete
   * @param {Function} recover function to be called on recover
   * @param {boolean} completeOnFirst completes on last or on first
   */
  function Limiter(limit, complete, recover, completeOnFirst) {
    _classCallCheck(this, Limiter);

    var _this = _possibleConstructorReturn(this, (Limiter.__proto__ || Object.getPrototypeOf(Limiter)).call(this, complete, recover, completeOnFirst));

    _this._limit = limit > 1 ? limit : 1;
    return _this;
  }

  /**
   * Returns the run condition
   *
   * @return {boolean} condition run condition
   * @protected
   */


  _createClass(Limiter, [{
    key: '_taskComplete',


    /**
     * Performs on every task complete
     *
     * @protected
     */
    value: function _taskComplete() {
      if (this.tasks.length > 0) {
        this._run(this.tasks.shift());
      }
    }
  }, {
    key: '_runCondition',
    get: function get() {
      return this.tasks.length > 0 && this._current.length < this._limit;
    }
  }]);

  return Limiter;
}(_Parallel3.default);

exports.default = Limiter;