'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Task2 = require('./Task');

var _Task3 = _interopRequireDefault(_Task2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * TimeoutTask
 *
 * Timeout task with run, complete and recover.
 * Executes after initial timeout
 *
 * @param {number} timeout wait before run
 * @param {Function} run function to be called on run
 * @param {Function} complete function to be called on complete
 * @param {Function} recover function to be called on recover
 */
var TimeoutTask = function (_Task) {
  _inherits(TimeoutTask, _Task);

  /**
   * TimeoutTask constructor
   *
   * @param {number} timeout wait before run
   * @param {Function} run function to be called on run
   * @param {Function} complete function to be called on complete
   * @param {Function} recover function to be called on recover
   */
  function TimeoutTask(timeout, run, complete, recover) {
    _classCallCheck(this, TimeoutTask);

    var _this = _possibleConstructorReturn(this, (TimeoutTask.__proto__ || Object.getPrototypeOf(TimeoutTask)).call(this, run, complete, recover));

    _this._timeout = timeout;
    _this._index = undefined;
    return _this;
  }

  /**
   * Runs a task
   *
   * Executes the run function if defined. All arguments will be
   * passed to it.
   *
   * @param {any} args any arguments
   */


  _createClass(TimeoutTask, [{
    key: 'run',
    value: function run() {
      var _this2 = this;

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      this._running = true;

      this._index = setTimeout(function () {
        var _get2;

        return (_get2 = _get(TimeoutTask.prototype.__proto__ || Object.getPrototypeOf(TimeoutTask.prototype), 'run', _this2)).call.apply(_get2, [_this2].concat(args));
      }, this._timeout);
    }

    /**
     * Completes a task
     *
     * Executes the complete function if defined. All arguments will be
     * passed to it.
     *
     * @param {any} args any arguments
     */

  }, {
    key: 'complete',
    value: function complete() {
      var _get3;

      this.stop();

      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      (_get3 = _get(TimeoutTask.prototype.__proto__ || Object.getPrototypeOf(TimeoutTask.prototype), 'complete', this)).call.apply(_get3, [this].concat(args));
    }

    /**
     * Stops a task
     *
     * @return {boolean} success
     */

  }, {
    key: 'stop',
    value: function stop() {
      if (this._running) {
        clearTimeout(this._index);

        this._index = undefined;

        return true;
      } else {
        return false;
      }
    }
  }]);

  return TimeoutTask;
}(_Task3.default);

exports.default = TimeoutTask;